import requests
from bs4 import BeautifulSoup
import csv
import re
from urllib.parse import urljoin, urldefrag
import os

def get_all_links(base_url):
    visited = set()
    to_visit = [base_url]
    links = set()
    
    while to_visit:
        url = to_visit.pop(0)
        if url in visited:
            continue
        visited.add(url)
        try:
            response = requests.get(url)
            response.raise_for_status()
            soup = BeautifulSoup(response.text, 'html.parser')
            for a in soup.find_all('a', href=True):
                full_url = urljoin(base_url, a['href'])
                full_url, _ = urldefrag(full_url)  # Remove the fragment part
                if full_url.startswith(base_url) and full_url not in visited and full_url not in links:
                    links.add(full_url)
                    to_visit.append(full_url)
            print(f"Visited: {url} - Links found: {len(links)}")
        except requests.RequestException as e:
            print(f"Failed to fetch {url}: {e}")
    return links

def format_links(links):
    formatted_links = []
    pattern = re.compile(r'https://oversealoan.com/index.php/\d{4}/\d{2}/\d{2}/([^/]+)/')

    for link in links:
        if 'index.php/' in link:
            new_link = pattern.sub(r'https://oversealoan.com/\1/', link)
            new_link = new_link.replace('index.php/', '')
            formatted_links.append((link, new_link))
        else:
            formatted_links.append((link, link))
    return formatted_links

def write_to_csv(formatted_links, file_path):
    os.makedirs(os.path.dirname(file_path), exist_ok=True)
    with open(file_path, mode='w', newline='', encoding='utf-8') as file:
        writer = csv.writer(file)
        writer.writerow(["source", "target", "regex", "code", "type", "hits", "title", "status"])
        for source, target in formatted_links:
            writer.writerow([source, target, 0, 301, "URL", 0, "", "enabled"])
    print(f"CSV file has been generated and saved to {file_path}")

def main():
    base_url = 'https://oversealoan.com'
    print(f"Starting to crawl links from {base_url}...")
    links = get_all_links(base_url)
    print(f"Total links found: {len(links)}")
    formatted_links = format_links(links)
    file_path = 'C:\\db\\redirection_rules.csv'
    write_to_csv(formatted_links, file_path)
    print('Process completed successfully.')

if __name__ == "__main__":
    main()
